# encoding: utf-8

# https://api.crunchbase.com/v/2/organization/#{permalink}/funding_rounds

module Crunchbase
  class FundingRound < CBEntity
    
    RESOURCE_NAME = 'funding-round'
    RESOURCE_LIST = 'funding_rounds'

    attr_reader :type_name, :name, :post_money_valuation_currency_code, :permalink, :funding_type, 
                :money_raised_usd, :announced_on_trust_code, :money_raised, 
                :money_raised_currency_code, :series, :announced_on, :canonical_currency_code,
                :created_at, :updated_at, :announced_on_year, :announced_on_day, :announced_on_month, 
                :pre_money_valuation_currency_code, :pre_money_valuation_usd, :pre_money_valuation, 
                :post_money_valuation_usd, :target_money_raised_usd

    attr_reader :investments, :funded_organizations, :new_items

    attr_reader :investments_total_items, :funded_organizations_total_items, :new_items_total_items

    def initialize(json)
      @type_name          = json['type']
      properties          = json['properties']
      relationships       = json['relationships']

      @name               = properties['name']
      @funding_type       = properties['funding_type']
      @permalink          = properties['permalink']
      @series             = properties['series']
      @money_raised_usd   = properties['money_raised_usd']
      
      if (properties['announced_on'].blank? || properties['announced_on'] == "0000-01-01")
        @announced_on = nil
      else
        @announced_on = DateTime.parse(properties['announced_on'])
      end
      @announced_on_year    = properties['announced_on_year']
      @announced_on_month   = properties['announced_on_month']
      @announced_on_day     = properties['announced_on_day']

      @announced_on_trust_code    = properties['announced_on_trust_code']
      @canonical_currency_code    = properties['canonical_currency_code']
      @money_raised               = properties['money_raised']
      @created_at                 = Time.at(properties['created_at']).utc
      @updated_at                 = Time.at(properties['updated_at']).utc
      @money_raised_currency_code           = properties['money_raised_currency_code']
      @post_money_valuation_currency_code   = properties['post_money_valuation_currency_code']

      @post_money_valuation_usd   = properties['post_money_valuation_usd']
      @target_money_raised_usd    = properties['target_money_raised_usd']
      @pre_money_valuation_usd    = properties['pre_money_valuation_usd']
      @pre_money_valuation        = properties['pre_money_valuation']
      @pre_money_valuation_currency_code   = properties['pre_money_valuation_currency_code']

      @investments_list           = relationships['investments']
      @funded_organizations_list  = relationships['funded_organization']
      @new_items_list             = relationships['news']
    end


    def investments
      @investments ||= Investment.array_from_list(@investments_list)
    end

    def funded_organizations
      @funded_organizations ||= Relationship.array_from_list(@funded_organizations_list)
    end

    def new_items
      @new_items ||= NewItem.array_from_list(@new_items_list)
    end

    def investments_total_items
      @investments_total_items ||= Investment.total_items_from_list(@investments_list)
    end

    def funded_organizations_total_items
      @funded_organizations_total_items ||= Relationship.total_items_from_list(@funded_organizations_list)
    end

    def new_items_total_items
      @new_items_total_items ||= NewItem.total_items_from_list(@new_items_list)
    end

  end
end
